/*
 * $Id: MoreFuncsVPI.java,v 1.2 2005/02/24 22:07:01 norman Exp $
 *
 * Copyright (C) 2004 by Brockmann Consult (info@brockmann-consult.de)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation. This program is distributed in the hope it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.bc.beam.visat;

import com.bc.jexp.EvalEnv;
import com.bc.jexp.EvalException;
import com.bc.jexp.Term;
import com.bc.jexp.impl.AbstractFunction;
import org.esa.beam.framework.dataop.barithm.BandArithmetic;
import org.esa.beam.visat.VisatApp;
import org.esa.beam.visat.VisatPlugIn;

import java.util.Random;

/**
 * This VISAT plugin adds more functions to VISAT's expression editor.
 *
 * @author Norman Fomferra
 */
public class MoreFuncsVPI implements VisatPlugIn {
    private static final double INV_LN_10 = 1.0 / Math.log(10.0);

    private Random _random = new Random();

    /**
     * Called by VISAT after the plug-in instance has been registered in VISAT's plug-in manager.
     *
     * @param visatApp a reference to the VISAT application instance.
     */
    public void initPlugIn(VisatApp visatApp) {

        BandArithmetic.registerFunction(new AbstractFunction.D("random_gaussian", 0) {
            public double evalD(EvalEnv env, Term[] args) throws EvalException {
                return _random.nextGaussian();
            }
        });

        BandArithmetic.registerFunction(new AbstractFunction.D("random_uniform", 0) {
            public double evalD(EvalEnv env, Term[] args) throws EvalException {
                return _random.nextDouble();
            }
        });

        BandArithmetic.registerFunction(new AbstractFunction.D("sinh", 1) {
            public double evalD(EvalEnv env, Term[] args) throws EvalException {
                return sinh(args[0].evalD(env));
            }
        });

        BandArithmetic.registerFunction(new AbstractFunction.D("cosh", 1) {
            public double evalD(EvalEnv env, Term[] args) throws EvalException {
                return cosh(args[0].evalD(env));
            }
        });

        BandArithmetic.registerFunction(new AbstractFunction.D("tanh", 1) {
            public double evalD(EvalEnv env, Term[] args) throws EvalException {
                return tanh(args[0].evalD(env));
            }
        });

        BandArithmetic.registerFunction(new AbstractFunction.D("sech", 1) {
            public double evalD(EvalEnv env, Term[] args) throws EvalException {
                return sech(args[0].evalD(env));
            }
        });

        BandArithmetic.registerFunction(new AbstractFunction.D("cosech", 1) {
            public double evalD(EvalEnv env, Term[] args) throws EvalException {
                return cosech(args[0].evalD(env));
            }
        });

        BandArithmetic.registerFunction(new AbstractFunction.D("log10", 1) {
            public double evalD(EvalEnv env, Term[] args) throws EvalException {
                return log10(args[0].evalD(env));
            }
        });
    }

    /**
     * Tells a plug-in to update its component tree (if any) since the Java look-and-feel has changed.
     * <p/>
     * <p>If a plug-in uses top-level containers such as dialogs or frames, implementors of this method should invoke
     * <code>SwingUtilities.updateComponentTreeUI()</code> on such containers.
     */
    public void updateComponentTreeUI() {
    }


    private static double sinh(final double x) {
        return 0.5 * (Math.exp(x) - Math.exp(-x));
    }

    private static double cosh(final double x) {
        return 0.5 * (Math.exp(x) + Math.exp(-x));
    }

    private static double tanh(final double x) {
        final double a = Math.exp(x);
        final double b = Math.exp(-x);
        return (a - b) / (a + b);
    }

    private static double sech(final double x) {
        return 2.0 / (Math.exp(x) + Math.exp(-x));
    }

    private static double cosech(final double x) {
        return 2.0 / (Math.exp(x) - Math.exp(-x));
    }

    private static double log10(final double x) {
        return Math.log(x) * INV_LN_10;
    }
}

